/*
 * Copyright 2014  Martin Klapetek <mklapetek@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef TIMEZONESI18N_H
#define TIMEZONESI18N_H

#include <QObject>
#include <QHash>
#include <QLocale>

class TimezonesI18n : public QObject
{
    Q_OBJECT

public:
    explicit TimezonesI18n(QObject *parent = nullptr);
    Q_INVOKABLE QString i18nCity(const QString &city);
    Q_INVOKABLE QString i18nContinents(const QString &continent);
    Q_INVOKABLE QString i18nCountry(QLocale::Country country);

private:
    void init();

    QHash<QString, QString> m_i18nCities;
    QHash<QString, QString> m_i18nContinents;
    QHash<QLocale::Country, QString> m_i18nCountries;
    bool m_isInitialized;
};

#endif // TIMEZONESI18N_H
